@@ -2,7 +2,7 @@
 
    This file is part of the lzop file compressor.
 
-   Copyright (C) 1996-2003 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2010 Markus Franz Xaver Johannes Oberhumer
    All Rights Reserved.
 
    lzop and the LZO library are free software; you can redistribute them
@@ -28,12 +28,17 @@
 
 #include "conf.h"
 
+#if (ACC_CC_MSC && (_MSC_VER >= 1000))
+   /* avoid '-W4' warnings */
+#  pragma warning(disable: 4244)
+#endif
+
 
 /*************************************************************************
 //
 **************************************************************************/
 
-void t_sub1(lzo_byte *p, lzo_uint l)
+void t_sub1(lzo_bytep p, lzo_uint l)
 {
     unsigned char b = 0;
 
@@ -44,7 +49,7 @@ void t_sub1(lzo_byte *p, lzo_uint l)
 }
 
 
-void t_add1(lzo_byte *p, lzo_uint l)
+void t_add1(lzo_bytep p, lzo_uint l)
 {
     unsigned char b = 0;
 
@@ -59,7 +64,7 @@ void t_add1(lzo_byte *p, lzo_uint l)
 //
 **************************************************************************/
 
-void t_sub(lzo_byte *p, lzo_uint l, int n)
+void t_sub(lzo_bytep p, lzo_uint l, int n)
 {
     unsigned char b[16];
     int i;
@@ -81,7 +86,7 @@ void t_sub(lzo_byte *p, lzo_uint l, int n)
 }
 
 
-void t_add(lzo_byte *p, lzo_uint l, int n)
+void t_add(lzo_bytep p, lzo_uint l, int n)
 {
     unsigned char b[16];
     int i;
@@ -107,16 +112,16 @@ void t_add(lzo_byte *p, lzo_uint l, int n)
 //
 **************************************************************************/
 
-void t_mtf(lzo_byte *p, lzo_uint l)
+void t_mtf(lzo_bytep p, lzo_uint l)
 {
     unsigned char b[256];
     unsigned char c;
-    int i;
+    unsigned i;
 
     if (l <= 1)
         return;
 
-    i = 255; do b[i] = (unsigned char) i; while (--i >= 0);
+    i = 256; do { --i; b[i] = (unsigned char) i; } while (i != 0);
 
     do {
         c = *p;
@@ -132,16 +137,16 @@ void t_mtf(lzo_byte *p, lzo_uint l)
 }
 
 
-void t_unmtf(lzo_byte *p, lzo_uint l)
+void t_unmtf(lzo_bytep p, lzo_uint l)
 {
     unsigned char b[256];
     unsigned char c;
-    int i;
+    unsigned i;
 
     if (l <= 1)
         return;
 
-    i = 255; do b[i] = (unsigned char) i; while (--i >= 0);
+    i = 256; do { --i; b[i] = (unsigned char) i; } while (i != 0);
 
     do {
         i = *p;
