@@ -2,7 +2,7 @@
 
    This file is part of the lzop file compressor.
 
-   Copyright (C) 1996-2003 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2010 Markus Franz Xaver Johannes Oberhumer
    All Rights Reserved.
 
    lzop and the LZO library are free software; you can redistribute them
@@ -38,7 +38,7 @@
 **************************************************************************/
 
 static void recolor_line(screen_t *screen, int frame, int line,
-                         lzo_byte *f, int len, int cy, int col)
+                         lzo_bytep f, int len, int cy, int col)
 {
     unsigned char colmap[4];
     int k;
@@ -82,7 +82,7 @@ static void recolor_line(screen_t *screen, int frame, int line,
 #if 1
     for (k = 0; k < len; k += 2)
     {
-        lzo_byte *p = &f[k+1];
+        lzo_bytep p = &f[k+1];
         if (*p != 0)
             *p = (colmap[ (*p >> 4) & 3 ] << 4) | colmap[ *p & 3 ];
     }
@@ -149,8 +149,8 @@ void screen_show_frames(screen_t *screen)
 {
     int i, j, k;
     lzo_uint out_len = UNCOMPRESSED_SIZE;
-    lzo_byte *frames = NULL;
-    lzo_byte *frames_mem = NULL;
+    lzo_bytep frames = NULL;
+    lzo_bytep frames_mem = NULL;
     int c_cx, c_cy;
     const int len = 79 * 2;         /* size of one line */
     const int nframes = 19;         /* number of frames */
@@ -169,7 +169,7 @@ void screen_show_frames(screen_t *screen)
         exit(EXIT_ERROR);
 #endif
 
-    frames = frames_mem = (lzo_byte *) malloc(UNCOMPRESSED_SIZE + 3);
+    frames = frames_mem = (lzo_bytep) malloc(UNCOMPRESSED_SIZE + 3);
     if (frames == NULL)
         return;
 
