@@ -2,7 +2,7 @@
 
    This file is part of the lzop file compressor.
 
-   Copyright (C) 1996-2003 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2010 Markus Franz Xaver Johannes Oberhumer
    All Rights Reserved.
 
    lzop and the LZO library are free software; you can redistribute them
@@ -47,12 +47,13 @@ void head(void)
 
     fg = con_fg(f,FG_GREEN);
     con_fprintf(f,
-                "                              Laetor Zonas Obteri\n"
-                "         Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003\n"
+                "                          Lempel-Ziv-Oberhumer Packer\n"
+                "                           Copyright (C) 1996 - 2010\n"
                 "lzop v%-11s  Markus Franz Xaver Johannes Oberhumer  %20s\n"
                 "\n",
                 LZOP_VERSION_STRING, LZOP_VERSION_DATE);
     fg = con_fg(f,fg);
+    UNUSED(fg);
 }
 
 
@@ -104,15 +105,17 @@ void help(void)
     fg = con_fg(f,FG_YELLOW);
     con_fprintf(f,"Commands:\n");
     fg = con_fg(f,fg);
+    con_fprintf(f,
 #if defined(USE_LZO1X_1_15) && defined(USE_LZO1X_999)
-    con_fprintf(f,"  -1     compress faster                   -9    compress better\n");
+"  -1     compress faster                   -9    compress better\n"
 #elif defined(USE_LZO1X_1_15)
-    con_fprintf(f,"  -1     compress faster\n");
+"  -1     compress faster\n"
 #endif
-    con_fprintf(f,"  -d     decompress                        -x    extract (same as -dPp)\n");
-    con_fprintf(f,"  -l     list compressed file              -I    display system information\n");
-    con_fprintf(f,"  -t     test compressed file              -V    display version number\n");
-    con_fprintf(f,"  -h     give this help                    -L    display software license\n");
+"  -d     decompress                        -x    extract (same as -dPp)\n"
+"  -l     list compressed file              -I    display system information\n"
+"  -t     test compressed file              -V    display version number\n"
+"  -h     give this help                    -L    display software license\n"
+);
 
     fg = con_fg(f,FG_YELLOW);
     con_fprintf(f,"Options:\n");
@@ -120,17 +123,21 @@ void help(void)
 
     con_fprintf(f,
 "  -q     be quiet                          -v       be verbose\n"
-"  -c     write on standard output          -oFILE   write output to `FILE'\n"
-"  -p     write output to current dir       -pDIR    write to path `DIR'\n"
+"  -c     write on standard output          -oFILE   write output to 'FILE'\n"
+"  -p     write output to current dir       -pDIR    write to path 'DIR'\n"
 "  -f     force overwrite of output files\n"
 "  -n     do not restore the original file name%s\n"
 "  -N     restore the original file name%s\n"
 "  -P     restore or save the original path and file name\n"
 "  -S.suf use suffix .suf on compressed files\n"
-"  -F     do *not* store or verify checksum of files (faster)\n"
+#if 0
+"  -F     do *not* store or verify checksum of files (a little bit faster)\n"
+#endif
 "  -U     delete input files after successful operation (like gzip and bzip2)\n"
 "  file.. files to (de)compress. If none given, try standard input.\n"
 , dn, dN);
+
+    UNUSED(fg);
 }
 
 
@@ -171,9 +178,14 @@ con_fprintf(f,
 //
 **************************************************************************/
 
-const char lzop_rcsid[] =
-    "\n$" "Id: lzop " LZOP_VERSION_STRING " built " __DATE__ " " __TIME__ " $\n";
+#ifndef LZOP_BUILD_DATE_TIME
+#define LZOP_BUILD_DATE_TIME __DATE__ " " __TIME__
+#endif
 
+const char lzop_rcsid[] =
+    "\n$" "Id: lzop " LZOP_VERSION_STRING " built " LZOP_BUILD_DATE_TIME " $\n";
+
+LZO_EXTERN(const lzo_bytep) lzo_copyright(void);
 
 void version(void)
 {
@@ -187,9 +199,9 @@ void version(void)
 
     con_fprintf(f,
         "lzop version: v" LZOP_VERSION_STRING ", " LZOP_VERSION_DATE "\n"
-        "lzop build date: " __DATE__ " " __TIME__ "\n");
+        "lzop build date: " LZOP_BUILD_DATE_TIME "\n");
 
-    for (i = 0, qq = __lzo_copyright; i < sizeof(pp)-1 && *qq; i++, qq++)
+    for (i = 0, qq = lzo_copyright(); i < sizeof(pp)-1 && *qq; i++, qq++)
         pp[i] = (char) *qq;
     pp[i] = 0;
     p = strstr(pp,"LZO version");
@@ -237,10 +249,10 @@ void sysinfo(void)
 #if defined(OPTIONS_VAR)
     env = getenv(OPTIONS_VAR);
     if (env && env[0])
-        con_fprintf(f,"Contents of environment variable %s: `%s'\n\n",
+        con_fprintf(f,"Contents of environment variable %s: '%s'\n\n",
                        OPTIONS_VAR, env);
     else
-        con_fprintf(f,"Environment variable `%s' is not set.\n\n",
+        con_fprintf(f,"Environment variable '%s' is not set.\n\n",
                        OPTIONS_VAR);
 #endif
 
