@@ -2,7 +2,7 @@
 
    This file is part of the lzop file compressor.
 
-   Copyright (C) 1996-2003 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2010 Markus Franz Xaver Johannes Oberhumer
    All Rights Reserved.
 
    lzop and the LZO library are free software; you can redistribute them
@@ -28,7 +28,6 @@
 
 #include "conf.h"
 #include "version.h"
-#include "mygetopt.h"
 
 
 /*************************************************************************
@@ -48,6 +47,7 @@ lzo_bool opt_file = 0;
 int opt_force = 0;
 lzo_bool opt_ignorewarn = 0;
 lzo_bool opt_keep = 0;
+int opt_num_threads = 1; /* NOT YET IMPLEMENTED */
 #ifdef MAINT
 int opt_noheader = 0;
 #endif
@@ -71,6 +71,7 @@ static int done_output_name = 0;
 static int done_output_path = 0;
 static int done_suffix = 0;
 
+/* invocation options */
 enum {
     PGM_LZOP,
     PGM_UNLZOP,
@@ -96,12 +97,12 @@ static file_t fo;
 
 static unsigned long total_d_files = 0;
 static unsigned long total_c_files = 0;
-static unsigned long total_d_len = 0;
-static unsigned long total_c_len = 0;
+static lzop_ulong_t total_d_len = 0;
+static lzop_ulong_t total_c_len = 0;
 
 /* some statistics */
-static unsigned long total_bytes_written = 0;
-static unsigned long total_bytes_read = 0;
+static lzop_ulong_t total_bytes_written = 0;
+static lzop_ulong_t total_bytes_read = 0;
 
 
 /*************************************************************************
@@ -198,19 +199,19 @@ void e_envopt(const char *n)
 {
     fflush(con_term);
     if (n)
-        fprintf(stderr,"%s: invalid string `%s' in environment variable `%s'\n",
+        fprintf(stderr,"%s: invalid string '%s' in environment variable '%s'\n",
                         argv0, n, OPTIONS_VAR);
     else
-        fprintf(stderr,"%s: illegal option in environment variable `%s'\n",
+        fprintf(stderr,"%s: illegal option in environment variable '%s'\n",
                         argv0, OPTIONS_VAR);
     e_exit(EXIT_USAGE);
 }
 #endif
 
 
-RETSIGTYPE SIGTYPEENTRY e_sighandler(int signum)
+RETSIGTYPE __acc_cdecl_sighandler e_sighandler(acc_signo_t signo)
 {
-    UNUSED(signum);
+    UNUSED(signo);
     e_exit(EXIT_FATAL);
 }
 
@@ -352,13 +353,16 @@ int f_open(file_t *ft, lzo_bool r)
 #if defined(O_BINARY)
     ft->open_flags |= O_BINARY;
 #endif
+#if (ACC_OS_WIN32 || ACC_OS_WIN64) && defined(_O_SEQUENTIAL)
+    ft->open_flags |= _O_SEQUENTIAL;
+#endif
 #if defined(USE_FOPEN)
     ft->file = NULL;
     if (r)
         ft->file = fopen(ft->name,"rb");
     else if (ft->open_flags & O_EXCL)
     {
-        if (file_exists(ft->name)
+        if (file_exists(ft->name))
             errno = EEXIST;
         else
             ft->file = fopen(ft->name,"wb");
@@ -372,7 +376,7 @@ int f_open(file_t *ft, lzo_bool r)
     }
 #else
     if (r)
-        ft->fd = open(ft->name, ft->open_flags);
+        ft->fd = open(ft->name, ft->open_flags, 0);
     else
     {
 #if defined(O_EXCL_BROKEN)
@@ -419,102 +423,36 @@ int f_close(file_t *ft)
 
 lzo_int read_buf(file_t *ft, lzo_voidp buffer, lzo_int cnt)
 {
-    lzo_bytep buf = (lzo_bytep) buffer;
-    lzo_int insize = 0;
+    lzo_int n;
+    long l;
 
-    errno = 0;
-    while (cnt > 0)
-    {
-#if defined(USE__READ)
-        lzo_int n;
-        n = _read(ft->fd, buf, cnt);
-#elif (ACC_HAVE_MM_HUGE_PTR)
-        long n;
-        n = acc_hread(ft->fd, buf, cnt);
-#elif (SIZE_T_MAX < LZO_UINT_MAX)
-        unsigned char b[2048];  /* don't waste stack on 16-bit systems */
-        unsigned char *bb;
-        int i;
-        int n;
+    assert(cnt >= 0 && cnt < LONG_MAX);
+    l = acc_safe_hread(ft->fd, buffer, (long) cnt);
+    n = (lzo_int) l;
+    assert(n >= 0); assert(n == l);
 
-        n = cnt > (lzo_int) sizeof(b) ? (int) sizeof(b) : (int) cnt;
-        n = read(ft->fd, b, n);
-        for (i = 0, bb = b; i < n; i++, bb++)
-            buf[i] = *bb;
-#else
-        lzo_int n;
-        n = read(ft->fd, buf, cnt);
-#endif
-        if (n == 0)
-            break;      /* end of file */
-        if (n < 0)
-        {
-#if defined(EINTR)
-            if (errno == EINTR)
-            {
-                errno = 0;
-                continue;
-            }
-#endif
-            break;      /* read error */
-        }
-        cnt -= n;
-        buf += n;
-        insize += n;
-        ft->bytes_read += n;
-        total_bytes_read += n;
-    }
-
-    return insize;
+    ft->bytes_read += n;
+    total_bytes_read += n;
+    return n;
 }
 
 
 void write_buf(file_t *ft, const lzo_voidp buffer, lzo_int cnt)
 {
-    const lzo_bytep buf = (const lzo_bytep) buffer;
+    lzo_int n;
+    long l;
 
+    assert(cnt >= 0 && cnt < LONG_MAX);
     if (ft->fd < 0)
         return;
+    l = acc_safe_hwrite(ft->fd, buffer, (long) cnt);
+    n = (lzo_int) l;
+    assert(n >= 0); assert(n == l);
 
-    errno = 0;
-    while (cnt > 0)
-    {
-#if defined(USE__WRITE)
-        lzo_int n;
-        n = _write(ft->fd, buf, cnt);
-#elif (ACC_HAVE_MM_HUGE_PTR)
-        long n;
-        n = acc_hwrite(ft->fd, buf, cnt);
-#elif (SIZE_T_MAX < LZO_UINT_MAX)
-        unsigned char b[2048];  /* don't waste stack on 16-bit systems */
-        unsigned char *bb;
-        int i;
-        int n;
-
-        n = cnt > (lzo_int) sizeof(b) ? (int) sizeof(b) : (int) cnt;
-        for (i = 0, bb = b; i < n; i++, bb++)
-            *bb = buf[i];
-        n = write(ft->fd, b, n);
-#else
-        lzo_int n;
-        n = write(ft->fd, buf, cnt);
-#endif
-        if (n < 0)
-        {
-#if defined(EINTR)
-            if (errno == EINTR)
-            {
-                errno = 0;
-                continue;
-            }
-#endif
-            write_error(ft);
-        }
-        cnt -= n;
-        buf += n;
-        ft->bytes_written += n;
-        total_bytes_written += n;
-    }
+    ft->bytes_written += n;
+    total_bytes_written += n;
+    if (n != cnt)
+        write_error(ft);
 }
 
 
@@ -558,7 +496,7 @@ static void set_be32(unsigned char *b, lzo_uint32 v)
 }
 
 
-#if 0
+#if 0 /* NOT USED */
 static void write8(file_t *ft, int v)
 {
     unsigned char b = (unsigned char) v;
@@ -667,7 +605,7 @@ static lzo_int f_read(file_t *ft, lzo_voidp buf, lzo_int cnt)
 
 
 /***********************************************************************
-// lzop file signature (shamelessly borrowed from PNG)
+// lzop file signature
 ************************************************************************/
 
 /*
@@ -695,7 +633,7 @@ static const char * const header_error[] = {
     "header corrupted (UNIX -> DOS conversion ?)",                      /*  8 */
     "header corrupted (end of line conversion ?)",                      /*  9 */
     "header corrupted (DOS EOF conversion ?)",                          /* 10 */
-    "header corrupted (transmitted through a 7bit channel ?)",          /* 11 */
+    "header corrupted (transmitted through a 7-bit channel ?)",         /* 11 */
     "header corrupted (transmitted in text mode ?)",                    /* 12 */
     "unknown header flags -- get a newer version of " PACKAGE,          /* 13 */
     "unknown compression method -- get a newer version of " PACKAGE,    /* 14 */
@@ -707,14 +645,16 @@ static const char * const header_error[] = {
 };
 
 
-/* This coding is derived from Alexander Lehmann's pngcheck code. */
-int check_magic(const unsigned char *magic)
+static int check_magic(const unsigned char *magic)
 {
     const unsigned char *m;
 
     if (memcmp(magic,lzop_magic,sizeof(lzop_magic)) == 0)
         return 0;
 
+    /* We have a bad magic signature. Try to figure what possibly
+     * could have gone wrong. */
+
     /* look at bytes 1-3: "LZO" in hex and local text format */
     if (memcmp(&magic[1],&lzop_magic[1],3) != 0 &&
         memcmp(&magic[1],"LZO",3) != 0)
@@ -802,23 +742,12 @@ void init_compress_header(header_t *h, const file_t *fip, const file_t *fop)
 
     h->mode = fix_mode_for_header(fip->st.st_mode);
 
-    if (fip->st.st_mtime != (time_t) -1)
+    if (fip->st.st_mtime)
     {
-        h->mtime = (lzo_uint32) fip->st.st_mtime;
-        if (h->mtime)
-        {
-#if 0 && defined(HAVE_LOCALTIME) && defined(HAVE_GMTIME)
-            /* difference of local timezone to GMT */
-            time_t t1, t2;
-            t1 = mktime(localtime(&fip->st.st_mtime));
-            t2 = mktime(gmtime(&fip->st.st_mtime));
-            if (t1 != (time_t) -1 && t2 != (time_t) -1)
-            {
-                h->gmtdiff = (lzo_uint32) ((lzo_int32) (t1 - t2));
-                h->flags |= F_H_GMTDIFF;
-            }
-#endif
-        }
+        h->mtime_low = (lzo_uint32) (fip->st.st_mtime);
+        h->mtime_high = (lzo_uint32) (fip->st.st_mtime >> 16 >> 16);
+        if ((lzo_int32) h->mtime_high < 0)
+            h->mtime_high = 0;
     }
 
     if (fip->name[0] && fip->fd != STDIN_FILENO)
@@ -827,7 +756,7 @@ void init_compress_header(header_t *h, const file_t *fip, const file_t *fop)
         if (opt_path)
         {
             char newname[255+1];
-            r = fn_cleanpath(fip->name, newname, sizeof(newname));
+            r = fn_cleanpath(fip->name, newname, sizeof(newname), 0);
             if (r > 0 && newname[0] && strlen(newname) <= 255)
             {
                 strcpy(h->name, newname);
@@ -850,10 +779,10 @@ void init_compress_header(header_t *h, const file_t *fip, const file_t *fop)
 
 void write_header(file_t *ft, const header_t *h)
 {
-    int l;
+    size_t l;
 
 #ifdef MAINT
-    /* undocumented option `--no-header' */
+    /* undocumented option '--no-header'. just for testing. */
     if (opt_noheader > 0)
     {
         switch (opt_noheader)
@@ -888,14 +817,14 @@ void write_header(file_t *ft, const header_t *h)
     if (h->flags & F_H_FILTER)
         f_write32(ft,h->filter);
     f_write32(ft,h->mode);
-    f_write32(ft,h->mtime);
-    f_write32(ft,h->gmtdiff);
+    f_write32(ft,h->mtime_low);
+    f_write32(ft,h->mtime_high);
 
-    l = (int) strlen(h->name);
-    assert(l >= 0 && l <= 255);
-    f_write8(ft,l);
-    if (l)
-        f_write(ft,h->name,l);
+    l = strlen(h->name);
+    assert(l <= 255);
+    f_write8(ft,(int)l);
+    if (l > 0)
+        f_write(ft,h->name,(int)l);
 
     if (h->flags & F_H_CRC32)
         f_write32(ft,ft->f_crc32);
@@ -911,7 +840,7 @@ static int read_header(file_t *ft, header_t *h)
     lzo_uint32 checksum;
 
     memset(h,0,sizeof(header_t));
-    h->version_needed_to_extract = 0x0900;  /* first lzop version */
+    h->version_needed_to_extract = 0x0900;  /* first public lzop version */
     h->level = 0;
     h->method_name = "unknown";
 
@@ -941,15 +870,24 @@ static int read_header(file_t *ft, header_t *h)
     if (h->flags & F_STDIN) /* do not use mode from stdin compression */
         h->mode = 0;
 #endif
-    f_read32(ft,&h->mtime);
+    f_read32(ft,&h->mtime_low);
     if (h->version >= 0x0940)
-        f_read32(ft,&h->gmtdiff);
+        f_read32(ft,&h->mtime_high);
+    if (h->version < 0x0120) {
+        if (h->mtime_low == 0xffffffffUL)
+            h->mtime_low = 0;
+        h->mtime_high = 0;
+    }
 
     l = f_read8(ft,NULL);
-    if (l > 0)
-        if (f_read(ft,h->name,l) != l)
+    if (l > 0) {
+        char name[255+1];
+        if (f_read(ft,name,l) != l)
             read_error(ft);
-    h->name[l] = 0;
+        name[l] = 0;
+        if (fn_cleanpath(name, h->name, 255+1, 1|2) < 0)
+            h->name[0] = 0;
+    }
 
     checksum = (h->flags & F_H_CRC32) ? ft->f_crc32 : ft->f_adler32;
     f_read32(ft,&h->header_checksum);
@@ -990,7 +928,7 @@ static int read_header(file_t *ft, header_t *h)
 
 
 /* return 0 for valid magic, -1 for EOF, or positive value for error */
-int p_magic(file_t *ft)
+static int p_magic(file_t *ft)
 {
     int r;
     lzo_int l;
@@ -999,7 +937,7 @@ int p_magic(file_t *ft)
     l = read_buf(ft,magic,sizeof(magic));
     if (ft->part > 0 && l <= 0)
         return -1;
-    if (l == sizeof(magic))
+    if (l == (lzo_int) sizeof(magic))
         r = check_magic(magic);
     else
         r = 1;
@@ -1032,7 +970,7 @@ garbage:
 }
 
 
-lzo_bool p_header(file_t *ft, header_t *h)
+static lzo_bool p_header(file_t *ft, header_t *h)
 {
     int r;
 
@@ -1059,7 +997,7 @@ lzo_bool p_header(file_t *ft, header_t *h)
 // test
 **************************************************************************/
 
-void do_test(const header_t *h, unsigned long d_len, unsigned long c_len)
+void do_test(const header_t *h, lzop_ulong_t d_len, lzop_ulong_t c_len)
 {
     total_d_files++;
     total_c_len += c_len;
@@ -1076,10 +1014,10 @@ void do_test_total(void)
         return;
 
     f = stderr;
-    fprintf(f,"%ld file%s successfully tested", total_c_files,
+    fprintf(f,"%lu file%s successfully tested", total_c_files,
         total_c_files == 1 ? " was" : "s were");
     if (total_c_files != total_d_files)
-        fprintf(f," [containing %ld files]", total_d_files);
+        fprintf(f," [containing %lu files]", total_d_files);
     fprintf(f,"\n");
     fflush(f);
 }
@@ -1089,19 +1027,60 @@ void do_test_total(void)
 // list a file
 **************************************************************************/
 
-static unsigned get_ratio (unsigned long size, unsigned long packedsize)
+static unsigned long get_ratio(lzop_ulong_t d_len, lzop_ulong_t c_len)
 {
-    unsigned n1, n2;
+    unsigned long n1 = 1000L * 1000L;
+    unsigned long n2 = 1;
+    const lzop_ulong_t umax = ~((lzop_ulong_t)0);
 
-    if (size <= 0)
-        return 100 * 100;
-    n1 = 10000; n2 = 1;
-    while (n1 > 1 && packedsize > (ULONG_MAX / n1))
+    if (d_len <= 0)
+        return c_len <= 0 ? 0ul : n1;
+    while (n1 > 1 && c_len > (umax / n1))
     {
         n1 /= 10;
         n2 *= 10;
     }
-    return (unsigned) ((packedsize * n1) / (size / n2)) + 5;
+    return (unsigned long) ((c_len * n1) / (d_len / n2));
+}
+
+
+static void pr_size(FILE *f, lzop_ulong_t a, lzop_ulong_t b, int flags)
+{
+    unsigned long ratio, r1, r2, al, bl;
+
+    ratio = (flags & 1) ? get_ratio(a, b) : get_ratio(b, a);
+    ratio += 500;   /* for rounding */
+    r1 = ratio / 10000;
+    r2 = (ratio % 10000) / 1000;
+
+#if (SIZEOF_LONG >= 8) || !defined(acc_int64l_t)
+    al = (unsigned long) a;
+    bl = (unsigned long) b;
+    fprintf(f,"%9lu %9lu %3lu.%01lu%%", al, bl, r1, r2);
+#else
+    al = (unsigned long) (a % 1000000000ul);
+    bl = (unsigned long) (b % 1000000000ul);
+    if (a == al && b == bl)
+    {
+        fprintf(f,"%9lu %9lu %3lu.%01lu%%", al, bl, r1, r2);
+    }
+    else if (a == al)
+    {
+        unsigned long bh = (unsigned long) (b / 1000000000ul);
+        fprintf(f,"%9lu %lu%09lu %3lu.%01lu%%", al, bh, bl, r1, r2);
+    }
+    else if (b == bl)
+    {
+        unsigned long ah = (unsigned long) (a / 1000000000ul);
+        fprintf(f,"%lu%09lu %9lu %3lu.%01lu%%", ah, al, bl, r1, r2);
+    }
+    else
+    {
+        unsigned long ah = (unsigned long) (a / 1000000000ul);
+        unsigned long bh = (unsigned long) (b / 1000000000ul);
+        fprintf(f,"%lu%09lu %lu%09lu %3lu.%01lu%%", ah, al, bh, bl, r1, r2);
+    }
+#endif
 }
 
 
@@ -1116,17 +1095,15 @@ static char *modestr(lzo_uint32 mode)
 }
 
 
-void do_list(const header_t *h, unsigned long d_len, unsigned long c_len)
+void do_list(const header_t *h, lzop_ulong_t d_len, lzop_ulong_t c_len)
 {
     FILE *f;
-    unsigned ratio;
     char s[40];
     time_t t;
 
     f = stdout;
     s[0] = 0;
-    t = (time_t) h->mtime;
-    ratio = get_ratio(d_len,c_len);
+    t = get_mtime(h);
 
     if (total_d_files == 0 && opt_verbose > 0)
     {
@@ -1161,8 +1138,7 @@ void do_list(const header_t *h, unsigned long d_len, unsigned long c_len)
     else if (opt_verbose >= 2)
     {
         fprintf(f,"%-11s ", h->method_name);
-        fprintf(f,"%9lu %9lu %3u.%01u%%", d_len, c_len,
-                   ratio / 100, (ratio % 100) / 10);
+        pr_size(f, d_len, c_len, 1);
         if (t)
             time2str(s,sizeof(s),&t);
         s[16] = 0;  /* cut off seconds */
@@ -1173,8 +1149,7 @@ void do_list(const header_t *h, unsigned long d_len, unsigned long c_len)
     else
     {
         fprintf(f,"%-11s ", h->method_name);
-        fprintf(f,"%9lu %9lu %3u.%01u%%", c_len, d_len,
-                   ratio / 100, (ratio % 100) / 10);
+        pr_size(f, c_len, d_len, 0);
         if (fo.name[0])
             fprintf(f," %s", fo.name);
     }
@@ -1191,7 +1166,6 @@ void do_list(const header_t *h, unsigned long d_len, unsigned long c_len)
 void do_list_total(void)
 {
     FILE *f;
-    unsigned ratio;
 
     if (total_d_files < 2 || opt_verbose == 0)
         return;
@@ -1199,23 +1173,19 @@ void do_list_total(void)
         return;
 
     f = stdout;
-    ratio = get_ratio(total_d_len,total_c_len);
-
     if (opt_verbose >= 2)
     {
         fprintf(f,"              -------   -------  ----- ");
         fprintf(f,"                   ----\n");
         fprintf(f,"%-11s ", "");
-        fprintf(f,"%9lu %9lu %3u.%01u%%", total_d_len, total_c_len,
-                   ratio / 100, (ratio % 100) / 10);
+        pr_size(f, total_d_len, total_c_len, 1);
         fprintf(f,"  %-16s", "");
         fprintf(f,"  %lu files\n", total_d_files);
     }
     else
     {
         fprintf(f,"%-11s ", "");
-        fprintf(f,"%9lu %9lu %3u.%01u%%", total_c_len, total_d_len,
-                   ratio / 100, (ratio % 100) / 10);
+        pr_size(f, total_c_len, total_d_len, 0);
         fprintf(f," (totals -- %lu files)\n", total_d_files);
     }
     fflush(f);
@@ -1223,10 +1193,10 @@ void do_list_total(void)
 
 
 /*************************************************************************
-// list a file similar to `ls -ln'
+// list a file similar to 'ls -ln'
 **************************************************************************/
 
-void do_ls(const header_t *h, unsigned long d_len, unsigned long c_len)
+void do_ls(const header_t *h, lzop_ulong_t d_len, lzop_ulong_t c_len)
 {
     FILE *f;
     char s[40];
@@ -1234,7 +1204,7 @@ void do_ls(const header_t *h, unsigned long d_len, unsigned long c_len)
     const char *name = fo.name[0] ? fo.name : UNKNOWN_NAME;
 
     f = stdout;
-    t = (time_t) h->mtime;
+    t = get_mtime(h);
     if (t == 0)
         time(&t);
     fprintf(f,"%-10s   1", modestr(h->mode));
@@ -1249,7 +1219,26 @@ void do_ls(const header_t *h, unsigned long d_len, unsigned long c_len)
         else
             fprintf(f," %-8ld", (long) fi.st.st_gid);
     }
-    fprintf(f," %8lu", (long) d_len);
+
+#if (SIZEOF_LONG >= 8) || !defined(acc_int64l_t)
+    fprintf(f," %8lu", (unsigned long) d_len);
+#else
+    {
+        unsigned long d0, d1, d2;
+        d0 = (unsigned long)  (d_len % 100000000ul);
+        if (d0 == d_len)
+            fprintf(f," %8lu", d0);
+        else
+        {
+            d1 = (unsigned long) ((d_len / 100000000ul) % 100000000ul);
+            d2 = (unsigned long) ((d_len / 100000000ul) / 100000000ul);
+            if (d2 != 0)
+                fprintf(f,"%lu%08lu%08lu", d2, d1, d0);
+            else
+                fprintf(f,"%lu%08lu", d1, d0);
+        }
+    }
+#endif
     time2ls(s, sizeof(s), &t);
     fprintf(f," %-12s",s);
     if (strchr(opt_ls_flags,'Q'))
@@ -1283,7 +1272,7 @@ static void print_os(FILE *f, lzo_uint32 flags)
 }
 
 
-void do_info(const header_t *h, unsigned long d_len, unsigned long c_len)
+void do_info(const header_t *h, lzop_ulong_t d_len, lzop_ulong_t c_len)
 {
     int v = opt_verbose;
     FILE *f;
@@ -1349,9 +1338,9 @@ static lzo_bool oname_error(void)
         return 1;
     }
     if (opt_name != 1)
-        error(&fi,"can't determine name of output file (try option `-N')");
+        error(&fi,"can't determine name of output file (try option '-N')");
     else
-        error(&fi,"can't determine name of output file (use option `-o')");
+        error(&fi,"can't determine name of output file (use option '-o')");
     strcpy(fo.name,UNKNOWN_NAME);
     return 0;
 }
@@ -1400,7 +1389,7 @@ static lzo_bool p_set_oname(const header_t *h)
     l = strlen(fo.name);
     if (l >= PATH_MAX)
     {
-        error(&fo,"name too long (use option `-o')");
+        error(&fo,"name too long (use option '-o')");
         return 0;
     }
     base = fo.name + fn_baseindex(fo.name);
@@ -1433,10 +1422,10 @@ static lzo_bool p_set_oname(const header_t *h)
 
         if (can_restore_name(&fi,h))
         {
-            if (!opt_path && (h->flags & F_H_PATH))
-                strcpy(base,fn_basename(h->name));
-            else
+            if (opt_path)
                 strcpy(base,h->name);
+            else
+                strcpy(base,fn_basename(h->name));
         }
         else if (opt_stdin)
         {
@@ -1486,7 +1475,7 @@ static lzo_bool p_set_oname(const header_t *h)
 
     if (strlen(fo.name) >= PATH_MAX)
     {
-        error(&fo,"name too long (use option `-o')");
+        error(&fo,"name too long (use option '-o')");
         return 0;
     }
 #if defined(DOSISH)
@@ -1563,7 +1552,7 @@ static lzo_bool open_stdin(file_t *ft)
     {
         if (acc_set_binmode(ft->fd, 1) == -1)
         {
-            p_fatal(ft,"setmode() failed");
+            p_fatal(ft,"acc_set_binmode(stdin) failed");
             return 0;
         }
     }
@@ -1578,6 +1567,8 @@ static lzo_bool open_stdin(file_t *ft)
             ft->st = st;
     }
 #endif
+    ft->st.st_atime = fix_time(ft->st.st_atime);
+    ft->st.st_mtime = fix_time(ft->st.st_mtime);
     return 1;
 }
 
@@ -1602,7 +1593,7 @@ static lzo_bool open_stdout(file_t *ft)
     {
         if (acc_set_binmode(ft->fd, 1) == -1)
         {
-            p_fatal(ft,"setmode() failed");
+            p_fatal(ft,"acc_set_binmode(stdout) failed");
             return 0;
         }
     }
@@ -1645,9 +1636,13 @@ lzo_bool p_open_fi(const char *name)
     if (opt_cmd == CMD_COMPRESS)
     {
         int suff = fn_has_suffix(fi.name);
+#if 1
+        if (opt_stdout || opt_output_name)
+            suff = SUFF_NONE;   /* do not warn */
+#endif
 #if 1
         if (opt_force >= 2)
-            suff = SUFF_NONE;
+            suff = SUFF_NONE;   /* do not warn */
 #else
         /* gzip: '--force' doesn't override these checks */
 #endif
@@ -1681,9 +1676,9 @@ lzo_bool p_open_fi(const char *name)
 /* open file */
     errno = 0;
     r = stat(fi.name, &fi.st);
+    saved_errno = errno;
     if (r != 0)
         memset(&fi.st, 0, sizeof(fi.st));
-    saved_errno = errno;
 #if defined(HAVE_LSTAT) && defined(S_ISLNK)
     r2 = lstat(fi.name, &fi.lst);
     if (r2 != 0)
@@ -1742,6 +1737,8 @@ lzo_bool p_open_fi(const char *name)
         return 0;
     }
 
+    fi.st.st_atime = fix_time(fi.st.st_atime);
+    fi.st.st_mtime = fix_time(fi.st.st_mtime);
     return 1;
 }
 
@@ -1861,7 +1858,9 @@ static void copy_perms(void)
 #endif
 #if defined(HAVE_CHOWN)
     /* copy the ownership */
-    (void) chown(fo.name, fi.st.st_uid, fi.st.st_gid);
+    if (chown(fo.name, fi.st.st_uid, fi.st.st_gid) != 0) {
+        /* ignore */
+    }
 #endif
 }
 
@@ -1904,13 +1903,14 @@ static void restore_perms(const header_t *h)
 #if defined(HAVE_UTIME)
     /* restore or copy the time stamp */
     struct utimbuf u;
-    if (opt_restore_time && h->mtime)
+    if (opt_restore_time && (h->mtime_low || h->mtime_high))
     {
-        u.actime = u.modtime = (time_t) h->mtime;
-        if (utime(fo.name,&u) != 0)
-            p_warn(&fo,"can't restore file time");
+        u.actime = u.modtime = get_mtime(h);
+        if (u.actime)
+            if (utime(fo.name,&u) != 0)
+                p_warn(&fo,"can't restore file time");
     }
-    else if (fi.st.st_atime > 0 && fi.st.st_mtime > 0)
+    else if (fi.st.st_atime && fi.st.st_mtime)
     {
         u.actime = fi.st.st_atime;
         u.modtime = fi.st.st_mtime;
@@ -1936,7 +1936,9 @@ static void restore_perms(const header_t *h)
 #if defined(HAVE_CHOWN)
     /* copy the ownership */
     if (!opt_stdin)
-        (void) chown(fo.name, fi.st.st_uid, fi.st.st_gid);
+        if (chown(fo.name, fi.st.st_uid, fi.st.st_gid) != 0) {
+            /* ignore */
+        }
 #endif
     UNUSED(h);
 }
@@ -1952,7 +1954,7 @@ static lzo_bool warn_multipart(file_t *ft, const header_t *h)
     if (opt_stdout || opt_output_name)
     {
         if (!ft->warn_multipart)
-            warn(&fi,"this is a multipart archive (try option `-N')");
+            warn(&fi,"this is a multipart archive (try option '-N')");
         ft->warn_multipart = 1;
     }
     else if (opt_file && !can_restore_name(ft,h))
@@ -1961,7 +1963,7 @@ static lzo_bool warn_multipart(file_t *ft, const header_t *h)
         if (opt_cmd == CMD_TEST)
         {
             if (!ft->warn_multipart)
-                warn(&fi,"this is a multipart archive (try option `-N')");
+                warn(&fi,"this is a multipart archive (try option '-N')");
             ft->warn_multipart = 1;
         }
         else if (can_restore_name(ft,h))
@@ -1972,7 +1974,7 @@ static lzo_bool warn_multipart(file_t *ft, const header_t *h)
         }
         else
         {
-            error(&fi,"multipart archive, but no filename stored (use option `-o')");
+            error(&fi,"multipart archive, but no filename stored (use option '-o')");
             return 0;
         }
     }
@@ -2013,7 +2015,7 @@ static lzo_bool do_decompress(const char *name, lzo_bool handle_perms)
 
         if (!p_header(&fi,&header))
         {
-            /* use `--info -f -f' to try to list a corrupted header */
+            /* use '--info -f -f' to try to list a corrupted header */
             if (opt_cmd == CMD_INFO && opt_force >= 2)
             {
                 (void) x_get_method(&header);
@@ -2142,7 +2144,7 @@ static void check_not_both(lzo_bool e1, lzo_bool e2, int c1, int c2)
     if (e1 && e2)
     {
         fprintf(stderr,"%s: ",argv0);
-        fprintf(stderr,"cannot use both `-%c' and `-%c'\n", c1, c2);
+        fprintf(stderr,"cannot use both '-%c' and '-%c'\n", c1, c2);
         e_usage();
     }
 }
@@ -2180,7 +2182,7 @@ void check_options(int i, int argc)
         opt_suffix[0] = 0;
         if (opt_unlink && !opt_force)
         {
-            fprintf(stderr,"%s: both `-c' and `-U' given (use `-f' to force)\n",argv0);
+            fprintf(stderr,"%s: both '-c' and '-U' given (use '-f' to force)\n",argv0);
             e_usage();
         }
     }
@@ -2197,7 +2199,7 @@ void check_options(int i, int argc)
     {
         if (opt_cmd == CMD_COMPRESS && opt_output_path)
         {
-            fprintf(stderr,"%s: cannot use `-p' when compressing stdin\n",argv0);
+            fprintf(stderr,"%s: cannot use '-p' when compressing stdin\n",argv0);
             e_usage();
         }
 
@@ -2305,7 +2307,7 @@ void set_output_name(const char *n, lzo_bool allow_m)
 #if 1
     if (done_output_name > 0)
     {
-        fprintf(stderr,"%s: option `-o' more than once given\n",argv0);
+        fprintf(stderr,"%s: option '-o' more than once given\n",argv0);
         e_usage();
     }
 #endif
@@ -2333,7 +2335,7 @@ void set_output_path(const char *n, lzo_bool allow_m)
 #if 1
     if (done_output_path > 0)
     {
-        fprintf(stderr,"%s: option `-p' more than once given\n",argv0);
+        fprintf(stderr,"%s: option '-p' more than once given\n",argv0);
         e_usage();
     }
 #endif
@@ -2389,20 +2391,20 @@ lzo_bool set_suffix(const char *n)
 #endif
           "rar", "raw", "sz", "tar", "taz", "tbz", "tgz", "tsz",
           "upx", "Z", "zip", "zoo", NULL };
-    const char * const *s;
+    const char * const *is = invalid_suffixes;
 
 #if 1
     if (done_suffix > 0)
     {
-        fprintf(stderr,"%s: option `-S' more than once given\n",argv0);
+        fprintf(stderr,"%s: option '-S' more than once given\n",argv0);
         e_usage();
         return 0;
     }
 #endif
-    while (*n == '.')
+    while (n && *n == '.')
         n++;
 
-    if (*n == 0 || *n == '-')
+    if (!n || *n == 0 || *n == '-')
         return 0;
 #if 1 || defined(DOSISH)
     if (strchr(n,'.'))
@@ -2411,14 +2413,14 @@ lzo_bool set_suffix(const char *n)
     for (p = n; *p; p++)
         if (strchr("+*?=/\\ \t\n\r\a", *p))
             return 0;
-    for (s = invalid_suffixes; *s; s++)
-        if (strcasecmp(n,*s) == 0)
+    for ( ; *is; is++)
+        if (strcasecmp(n,*is) == 0)
             return 0;
 
     l = strlen(n);
     if (l + 1 > SUFFIX_MAX || (opt_shortname && l > 3))
     {
-        fprintf(stderr,"%s: suffix `%s' is too long\n",argv0,n);
+        fprintf(stderr,"%s: suffix '%s' is too long\n",argv0,n);
         e_usage();
         return 0;
     }
@@ -2436,7 +2438,7 @@ lzo_bool set_suffix(const char *n)
 
 static
 char* prepare_shortopts(char *buf, const char *n,
-                        const struct mfx_option *longopts)
+                        const struct acc_getopt_longopt_t *longopts)
 {
     char *o = buf;
 
@@ -2461,8 +2463,9 @@ char* prepare_shortopts(char *buf, const char *n,
 }
 
 
-static int do_option(int optc)
+static int do_option(acc_getopt_p g, int optc)
 {
+#define mfx_optarg      g->optarg
     int i = 0;
     int m = -1;
 
@@ -2542,18 +2545,34 @@ static int do_option(int optc)
     case 'S':
         if (!set_suffix(mfx_optarg))
         {
-            fprintf(stderr,"%s: invalid suffix `%s'\n",argv0,mfx_optarg);
+            fprintf(stderr,"%s: invalid suffix '%s'\n",argv0,mfx_optarg);
             e_usage();
         }
         break;
     case 't':
         set_cmd(CMD_TEST);
         break;
+    case 'T':
+        if (!(mfx_optarg && isdigit(mfx_optarg[0])))
+        {
+            fprintf(stderr,"%s: invalid '--threads=' args: '%s'\n",argv0,mfx_optarg);
+            e_usage();
+        }
+        opt_num_threads = atoi(mfx_optarg);
+        if (opt_num_threads < 1 || opt_num_threads > MAX_NUM_THREADS)
+        {
+            fprintf(stderr,"%s: invalid number of threads: %d\n",argv0,opt_num_threads);
+            e_usage();
+        }
+#if !defined(WITH_THREADS)
+        opt_num_threads = 1;
+#endif
+        break;
     case 'U':
         opt_unlink = 1;
         break;
     case 'v':
-        opt_verbose = (opt_verbose < 2 ? 2 : opt_verbose + 1);
+        opt_verbose = (opt_verbose < 2) ? 2 : opt_verbose + 1;
         break;
     case 'V':
         set_cmd(CMD_VERSION);
@@ -2564,7 +2583,7 @@ static int do_option(int optc)
         opt_console = CON_NONE;
         fprintf(stdout,"lzop %s\n",LZOP_VERSION_STRING);
         fprintf(stdout,"LZO library %s\n",lzo_version_string());
-        fprintf(stdout,"Copyright (C) 1996-2003 Markus Franz Xaver Johannes Oberhumer\n");
+        fprintf(stdout,"Copyright (C) 1996-2010 Markus Franz Xaver Johannes Oberhumer\n");
         e_exit(EXIT_OK);
         break;
     case 'x':
@@ -2588,7 +2607,7 @@ static int do_option(int optc)
             for (i = 0; opt_ls_flags[i]; i++)
                 if (!strchr("FGQ",opt_ls_flags[i]))
                 {
-                    fprintf(stderr,"%s: invalid ls flags `%s'\n",argv0,mfx_optarg);
+                    fprintf(stderr,"%s: invalid '--ls' flags: '%s'\n",argv0,mfx_optarg);
                     e_usage();
                 }
         }
@@ -2675,7 +2694,10 @@ static int do_option(int optc)
 
     case 521:
         if (!(mfx_optarg && isdigit(mfx_optarg[0])))
+        {
+            fprintf(stderr,"%s: invalid '--filter=' args: '%s'\n",argv0,mfx_optarg);
             e_usage();
+        }
         if (strcmp(mfx_optarg,"0") == 0)
         {
             opt_filter = 0;
@@ -2683,7 +2705,10 @@ static int do_option(int optc)
         }
         opt_filter = atoi(mfx_optarg);
         if (opt_filter < 1 || opt_filter > 16)
+        {
+            fprintf(stderr,"%s: invalid filter: %d\n",argv0,opt_filter);
             e_usage();
+        }
         break;
 
     case '\0':
@@ -2698,13 +2723,27 @@ static int do_option(int optc)
     UNUSED(i);
     UNUSED(m);
     return 0;
+#undef mfx_optarg
+}
+
+
+static void handle_opterr(acc_getopt_p g, const char *f, void *v)
+{
+    struct A { va_list ap; };
+    struct A *a = (struct A *) v;
+    fprintf( stderr, "%s: ", g->progname);
+    if (a)
+        vfprintf(stderr, f, a->ap);
+    else
+        fprintf( stderr, "UNKNOWN GETOPT ERROR");
+    fprintf( stderr, "\n");
 }
 
 
 static int get_options(int argc, char **argv)
 {
 
-static const struct mfx_option longopts[] =
+static const struct acc_getopt_longopt_t longopts[] =
 {
     {"best",       0, 0, '9'},      /* compress better */
     {"decompress", 0, 0, 'd'},      /* decompress */
@@ -2751,6 +2790,7 @@ static const struct mfx_option longopts[] =
     {"silent",     0, 0, 'q'},      /* quiet mode */
     {"stdout",     0, 0, 'c'},      /* write output on standard output */
     {"suffix",     1, 0, 'S'},      /* use given suffix instead of .lzo */
+    {"threads", 0x21, 0, 'T'},      /* number of threads */
     {"to-stdout",  0, 0, 'c'},      /* write output on standard output */
     {"unlink",     0, 0, 'U'},
     {"verbose",    0, 0, 'v'},      /* verbose mode */
@@ -2763,27 +2803,29 @@ static const struct mfx_option longopts[] =
     { 0, 0, 0, 0 }
 };
 
+    acc_getopt_t mfx_getopt;
     int optc;
     int i;
-    char buf[128];
+    char shortopts[128];
 
-    prepare_shortopts(buf,"123456789hH?PVp::",longopts),
-    mfx_optind = 0;
-    mfx_opterr = 1;
-    while ((optc = mfx_getopt_long(argc, argv, buf, longopts, NULL)) >= 0)
+    prepare_shortopts(shortopts, "123456789hH?PVp::", longopts),
+    acc_getopt_init(&mfx_getopt, 1, argc, argv);
+    mfx_getopt.progname = argv0;
+    mfx_getopt.opterr = handle_opterr;
+    while ((optc = acc_getopt(&mfx_getopt, shortopts, longopts, NULL)) >= 0)
     {
-        if (do_option(optc) != 0)
+        if (do_option(&mfx_getopt, optc) != 0)
             e_usage();
     }
 
     /* accept "-" as synonym for stdin */
-    for (i = mfx_optind; i < argc; i++)
-        if (strcmp(argv[i],"-") == 0)
+    for (i = mfx_getopt.optind; i < argc; i++)
+        if (strcmp(argv[i], "-") == 0)
             opt_stdin = OPT_STDIN_REQUESTED;
-    while (mfx_optind < argc && strcmp(argv[mfx_optind],"-") == 0)
-        mfx_optind++;
-
-    return mfx_optind;
+    for (i = mfx_getopt.optind; i < argc; i++)
+        if (strcmp(argv[i], "-") != 0)
+            break;
+    return i;
 }
 
 
@@ -2793,7 +2835,7 @@ static void get_envoptions(int argc, char **argv)
 
 /* only some options are allowed in the environment variable */
 
-static const struct mfx_option longopts[] =
+static const struct acc_getopt_longopt_t longopts[] =
 {
     {"best",       0, 0, '9'},      /* compress better */
     {"checksum",   0, 0, 'C'},
@@ -2811,6 +2853,7 @@ static const struct mfx_option longopts[] =
     {"no-warn",    0, 0, 525},      /* do not display any warnings */
     {"quiet",      0, 0, 'q'},      /* quiet mode */
     {"silent",     0, 0, 'q'},      /* quiet mode */
+    {"threads", 0x21, 0, 'T'},      /* number of threads */
     {"unlink",     0, 0, 'U'},
     {"verbose",    0, 0, 'v'},      /* verbose mode */
 
@@ -2826,7 +2869,8 @@ static const struct mfx_option longopts[] =
     int nargc;
     char **nargv = NULL;
     static const char sep[] = " \t";
-    char buf[128];
+    acc_getopt_t mfx_getopt;
+    char shortopts[128];
 
     env = (char *) getenv(OPTIONS_VAR);
     if (env == NULL || !env[0])
@@ -2889,17 +2933,18 @@ static const struct mfx_option longopts[] =
         if (nargv[i][0] != '-' || !nargv[i][1] || strcmp(nargv[i],"--") == 0)
             e_envopt(nargv[i]);
 
-    prepare_shortopts(buf,"123456789P",longopts);
-    mfx_optind = 0;
-    mfx_opterr = 1;
-    while ((optc = mfx_getopt_long(nargc, nargv, buf, longopts, NULL)) >= 0)
+    prepare_shortopts(shortopts, "123456789P", longopts);
+    acc_getopt_init(&mfx_getopt, 1, nargc, nargv);
+    mfx_getopt.progname = argv0;
+    mfx_getopt.opterr = handle_opterr;
+    while ((optc = acc_getopt(&mfx_getopt, shortopts, longopts, NULL)) >= 0)
     {
-        if (do_option(optc) != 0)
+        if (do_option(&mfx_getopt, optc) != 0)
             e_envopt(NULL);
     }
 
-    if (mfx_optind < nargc)
-        e_envopt(nargv[mfx_optind]);
+    if (mfx_getopt.optind < nargc)
+        e_envopt(nargv[mfx_getopt.optind]);
 
     free(nargv);
     free(env);
@@ -2911,16 +2956,20 @@ static const struct mfx_option longopts[] =
 #endif /* defined(OPTIONS_VAR) */
 
 
-static void first_options(int argc, char **argv)
-{
-    int i;
+#define ACC_WANT_ACC_CHK_CH 1
+#undef ACCCHK_ASSERT
+#include "miniacc.h"
+#undef ACCCHK_ASSERT
 
-    for (i = 1; i < argc; i++)
-        if (strcmp(argv[i],"--version") == 0)
-            do_option('V'+256);
-    for (i = 1; i < argc; i++)
-        if (strcmp(argv[i],"--help") == 0)
-            do_option('h'+256);
+static void sanity_check(void)
+{
+#if (ACC_CC_MSC && ((_MSC_VER) < 700))
+#else
+#define ACCCHK_ASSERT(expr)     ACC_COMPILE_TIME_ASSERT(expr)
+#include "miniacc.h"
+#endif
+#undef ACCCHK_ASSERT
+#undef ACC_WANT_ACC_CHK_CH
 }
 
 
@@ -2928,22 +2977,21 @@ static void first_options(int argc, char **argv)
 // main entry point
 **************************************************************************/
 
-int main(int argc, char *argv[])
+int __acc_cdecl_main main(int argc, char *argv[])
 {
     int i;
     lzo_bool foreground = 0;
     static char default_argv0[] = "lzop";
     int cmdline_cmd = CMD_NONE;
 
-#if defined(__EMX__)
-    _response(&argc,&argv);
-    _wildcard(&argc,&argv);
-#endif
+    sanity_check();
+
 #if defined(__MINT__)
     __binmode(1);
     __set_binmode(stdout, 0);
     __set_binmode(stderr, 0);
 #endif
+    acc_wildargv(&argc, &argv);
 
 
 #if defined(__DJGPP__)
@@ -2952,7 +3000,7 @@ int main(int argc, char *argv[])
     opt_shortname = 1;
 #endif
 
-    current_time = time(NULL);
+    current_time = fix_time(time(NULL));
 
     if (!argv[0] || !argv[0][0])
         argv[0] = default_argv0;
@@ -2980,24 +3028,22 @@ int main(int argc, char *argv[])
 #ifdef SIGINT
     foreground = signal(SIGINT, SIG_IGN) != SIG_IGN;
     if (foreground)
-        (void) signal(SIGINT, (sig_type)e_sighandler);
+        (void) signal(SIGINT, e_sighandler);
 #endif
 #ifdef SIGBREAK
     if (foreground)
-        (void) signal(SIGBREAK, (sig_type)e_sighandler);
+        (void) signal(SIGBREAK, e_sighandler);
 #endif
 #ifdef SIGTERM
     if (signal(SIGTERM, SIG_IGN) != SIG_IGN)
-        (void) signal(SIGTERM, (sig_type)e_sighandler);
+        (void) signal(SIGTERM, e_sighandler);
 #endif
 #ifdef SIGHUP
     if (signal(SIGHUP, SIG_IGN) != SIG_IGN)
-        (void) signal(SIGHUP,  (sig_type)e_sighandler);
+        (void) signal(SIGHUP,  e_sighandler);
 #endif
 
-#if defined(HAVE_GETUMASK)
-    u_mask = (MODE_T) getumask();
-#elif defined(HAVE_UMASK)
+#if defined(HAVE_UMASK)
     u_mask = (MODE_T) umask(0700);
     (void) umask(u_mask);
 #endif
@@ -3012,6 +3058,7 @@ int main(int argc, char *argv[])
             fprintf(stderr,"library version conflict (%x, %x) - check your LZO installation !\n", LZO_VERSION, lzo_version());
         e_exit(EXIT_LZO_INIT);
     }
+#if 0
     if (lzo_version() < LZO_VERSION)
     {
         head();
@@ -3019,6 +3066,7 @@ int main(int argc, char *argv[])
         e_exit(EXIT_LZO_INIT);
     }
 #endif
+#endif
 #if defined(WITH_NRV)
     if (nrv_init() != NRV_E_OK)
     {
@@ -3036,7 +3084,6 @@ int main(int argc, char *argv[])
     assert(STDERR_FILENO >= 0);
 
     f_init();
-    first_options(argc,argv);
 #if defined(OPTIONS_VAR)
     get_envoptions(argc,argv);
 #endif
