@@ -2,7 +2,7 @@
 
    This file is part of the lzop file compressor.
 
-   Copyright (C) 1996-2003 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2010 Markus Franz Xaver Johannes Oberhumer
    All Rights Reserved.
 
    lzop and the LZO library are free software; you can redistribute them
@@ -33,68 +33,69 @@
 //
 **************************************************************************/
 
-static void do_init(memblock_p m, lzo_uint32 size, lzo_uint align)
+static void do_init(mblock_p m, lzo_uint32 size, lzo_uint align)
 {
     memset(m,0,sizeof(*m));
-    m->size = size;
-    m->align = (align > 1) ? align : 1;
-    assert((m->align & (m->align - 1)) == 0);
+    m->mb_size = size;
+    m->mb_align = (align > 1) ? align : 1;
+    assert((m->mb_align & (m->mb_align - 1)) == 0);
+    m->mb_adler32 = ADLER32_INIT_VALUE;
+    m->mb_crc32 = CRC32_INIT_VALUE;
 }
 
 
-lzo_bool mb_init(memblock_p m, lzo_uint32 size, lzo_uint align,
+#if 0
+lzo_bool mb_init(mblock_p m, lzo_uint32 size, lzo_uint align,
                  lzo_voidp heap, lzo_uint32 heap_size)
 {
     do_init(m,size,align);
-    if (m->size == 0)
+    if (m->mb_size == 0)
         return 1;
 
     if (heap == 0)
         return 0;
-    m->mem_alloc = (lzo_bytep) heap;
-    m->size_alloc = heap_size;
-    assert(m->size_alloc >= m->size + m->align - 1);
+    m->mb_mem_alloc = (lzo_bytep) heap;
+    m->mb_size_alloc = heap_size;
+    assert(m->mb_size_alloc >= m->mb_size + m->mb_align - 1);
 
-    m->mem = LZO_PTR_ALIGN_UP(m->mem_alloc,m->align);
-    assert(m->mem >= m->mem_alloc);
-    assert(m->mem + m->size <= m->mem_alloc + m->size_alloc);
+    m->mb_mem = LZO_PTR_ALIGN_UP(m->mb_mem_alloc,m->mb_align);
+    assert(m->mb_mem >= m->mb_mem_alloc);
+    assert(m->mb_mem + m->mb_size <= m->mb_mem_alloc + m->mb_size_alloc);
 #if 0
-    printf("m_init: %p %p %8ld %8ld %8ld\n", m->mem_alloc, m->mem,
-           (long) m->size_alloc, (long) m->size, (long) m->align);
+    printf("m_init: %p %p %8ld %8ld %8ld\n", m->mb_mem_alloc, m->mb_mem,
+           (long) m->mb_size_alloc, (long) m->mb_size, (long) m->mb_align);
 #endif
     return 1;
 }
+#endif
 
 
-lzo_bool mb_alloc(memblock_p m, lzo_uint32 size, lzo_uint align)
+lzo_bool mb_alloc(mblock_p m, lzo_uint32 size, lzo_uint align)
 {
     do_init(m,size,align);
-    if (m->size == 0)
+    if (m->mb_size == 0)
         return 1;
 
-    m->size_alloc = m->size + m->align - 1;
-    m->mem_alloc = (lzo_byte *) malloc(m->size_alloc);
-    if (m->mem_alloc == NULL)
+    m->mb_size_alloc = m->mb_size + m->mb_align - 1;
+    m->mb_mem_alloc = (lzo_bytep) acc_halloc(m->mb_size_alloc);
+    if (m->mb_mem_alloc == NULL)
         return 0;
-    m->flags = 1;
+    acc_hmemset(m->mb_mem_alloc, 0, m->mb_size_alloc);
 
-    m->mem = LZO_PTR_ALIGN_UP(m->mem_alloc,m->align);
-    assert(m->mem >= m->mem_alloc);
-    assert(m->mem + m->size <= m->mem_alloc + m->size_alloc);
+    m->mb_mem = LZO_PTR_ALIGN_UP(m->mb_mem_alloc,m->mb_align);
+    assert(m->mb_mem >= m->mb_mem_alloc);
+    assert(m->mb_mem + m->mb_size <= m->mb_mem_alloc + m->mb_size_alloc);
 #if 0
-    printf("m_alloc: %p %p %8ld %8ld %8ld\n", m->mem_alloc, m->mem,
-           (long) m->size_alloc, (long) m->size, (long) m->align);
+    printf("m_alloc: %p %p %8ld %8ld %8ld\n", m->mb_mem_alloc, m->mb_mem,
+           (long) m->mb_size_alloc, (long) m->mb_size, (long) m->mb_align);
 #endif
     return 1;
 }
 
 
-void mb_free(memblock_p m)
+void mb_free(mblock_p m)
 {
-    if (m->flags & 1)
-    {
-        free(m->mem_alloc);
-    }
+    acc_hfree(m->mb_mem_alloc);
     memset(m,0,sizeof(*m));
 }
 
